import numpy as np
import matplotlib.pyplot as plt
from functools import wraps

from MPSPlots.styles import mps as plot_style


def post_mpl_plot(function):
    extra_doc = """
    Extra Parameters
    ----------------
    show : bool
        Whether to display the plot after creation.
    save_as : str, optional
        If provided, save the figure to this path.
    figure_size : tuple, optional
        Size of the figure in inches.
    tight_layout : bool, default=True
        Whether to use tight layout for the figure.
    axes : matplotlib.axes.Axes, optional
        Axes to draw on. If None, a new figure+axes are created.
    xscale : str, optional
        X-axis scale type (e.g., 'linear', 'log').
    yscale : str, optional
        Y-axis scale type (e.g., 'linear', 'log').
    xlim : tuple, optional
        X-axis limits as (min, max).
    ylim : tuple, optional
        Y-axis limits as (min, max).
    style : str, default=plot_style
        Matplotlib style to use for the plot.
    **kwargs
        Additional keyword arguments to pass to the plotting function.

    Returns
    -------
    Figure : matplotlib.figure.Figure
        The figure with the market prices plot.
    """
    @wraps(function)
    def wrapper(
        *args,
        tight_layout: bool = True,
        save_as: str = None,
        show: bool = True,
        xscale: str = None,
        yscale: str = None,
        xlim: tuple = None,
        ylim: tuple = None,
        figure_size: tuple = None,
        style: str = plot_style,
        **kwargs):


        with plt.style.context(style):
            figure = function(*args, **kwargs)

            # Apply axis customizations
            axes = np.ravel(figure.axes)

            all_axes = axes if isinstance(axes, np.ndarray) else [axes]
            for ax in all_axes:
                if xscale:
                    ax.set_xscale(xscale)
                if yscale:
                    ax.set_yscale(yscale)
                if xlim:
                    ax.set_xlim(*xlim)
                if ylim:
                    ax.set_ylim(*ylim)

            if figure_size is not None:
                figure.set_figwidth(figure_size[0])
                figure.set_figheight(figure_size[1])

            if tight_layout:
                figure.tight_layout()

            if save_as is not None:
                figure.savefig(save_as, dpi=300)

            if show:
                plt.show()

            return figure

    # merge docstrings (original + extra)
    if function.__doc__:
        wrapper.__doc__ = f"{function.__doc__}\n{extra_doc}"
    else:
        wrapper.__doc__ = extra_doc
    return wrapper


def pre_plot(nrows: int = 1, ncols: int = 1, subplot_kw: dict = {}, gridspec_kw: dict = {}, sharex: bool = False, sharey: bool = False):
    """
    Decorator factory that creates a matplotlib figure with subplots
    before calling the decorated plotting function.

    Parameters
    ----------
    nrows : int
        Number of subplot rows.
    ncols : int
        Number of subplot columns.
    """
    def decorator(function):
        extra_doc = """
        Extra Parameters
        ----------------
        show : bool
            Whether to display the plot after creation.
        save_as : str, optional
            If provided, save the figure to this path.
        figure_size : tuple, optional
            Size of the figure in inches.
        tight_layout : bool, default=True
            Whether to use tight layout for the figure.
        axes : matplotlib.axes.Axes, optional
            Axes to draw on. If None, a new figure+axes are created.
        xscale : str, optional
            X-axis scale type (e.g., 'linear', 'log').
        yscale : str, optional
            Y-axis scale type (e.g., 'linear', 'log').
        xlim : tuple, optional
            X-axis limits as (min, max).
        ylim : tuple, optional
            Y-axis limits as (min, max).
        style : str, default=plot_style
            Matplotlib style to use for the plot.
        **kwargs
            Additional keyword arguments to pass to the plotting function.

        Returns
        -------
        axes : matplotlib.axes.Axes
            The axes with the market prices plot.
        """

        @wraps(function)
        def wrapper(*args,
                    show: bool = True,
                    save_as: str = None,
                    figure_size: tuple = None,
                    tight_layout: bool = True,
                    axes: plt.Axes = None,
                    xscale: str = None,
                    yscale: str = None,
                    xlim: tuple = None,
                    ylim: tuple = None,
                    style: str = plot_style,
                    **kwargs):

            with plt.style.context(style):
                if axes is None:
                    figure, axes = plt.subplots(
                        nrows=nrows,
                        ncols=ncols,
                        figsize=figure_size,
                        squeeze=False,
                        subplot_kw=subplot_kw,
                        gridspec_kw=gridspec_kw,
                        sharex=sharex,
                        sharey=sharey
                    )
                else:
                    figure = axes.get_figure()

                axes = np.array(axes).flatten()
                if nrows * ncols == 1:
                    axes = axes[0]

                # Call the decorated function
                if args and hasattr(args[0], "__class__"):
                    function(args[0], axes=axes, *args[1:], **kwargs)
                else:
                    function(*args, axes=axes, **kwargs)

                # Apply axis customizations
                all_axes = axes if isinstance(axes, np.ndarray) else [axes]
                for ax in all_axes:
                    if xscale:
                        ax.set_xscale(xscale)
                    if yscale:
                        ax.set_yscale(yscale)
                    if xlim:
                        ax.set_xlim(*xlim)
                    if ylim:
                        ax.set_ylim(*ylim)

                if tight_layout:
                    figure.tight_layout()

                if save_as is not None:
                    figure.savefig(save_as, dpi=300)

                if show:
                    plt.show()

                return figure

        # merge docstrings (original + extra)
        if function.__doc__:
            wrapper.__doc__ = f"{function.__doc__}\n{extra_doc}"
        else:
            wrapper.__doc__ = extra_doc
        return wrapper


    return decorator


def pre_figure_plot():
    """
    Decorator factory that creates a matplotlib figure with subplots
    before calling the decorated plotting function.

    Parameters
    ----------
    nrows : int
        Number of subplot rows.
    ncols : int
        Number of subplot columns.
    """
    def decorator(function):
        extra_doc = """
        Extra Parameters
        ----------------
        show : bool
            Whether to display the plot after creation.
        save_as : str, optional
            If provided, save the figure to this path.
        figure_size : tuple, optional
            Size of the figure in inches.
        tight_layout : bool, default=True
            Whether to use tight layout for the figure.
        axes : matplotlib.axes.Axes, optional
            Axes to draw on. If None, a new figure+axes are created.
        xscale : str, optional
            X-axis scale type (e.g., 'linear', 'log').
        yscale : str, optional
            Y-axis scale type (e.g., 'linear', 'log').
        xlim : tuple, optional
            X-axis limits as (min, max).
        ylim : tuple, optional
            Y-axis limits as (min, max).
        style : str, default=plot_style
            Matplotlib style to use for the plot.
        **kwargs
            Additional keyword arguments to pass to the plotting function.

        Returns
        -------
        axes : matplotlib.axes.Axes
            The axes with the market prices plot.
        """

        @wraps(function)
        def wrapper(*args,
                    show: bool = True,
                    save_as: str = None,
                    figure_size: tuple = None,
                    tight_layout: bool = True,
                    axes: plt.Axes = None,
                    xscale: str = None,
                    yscale: str = None,
                    xlim: tuple = None,
                    ylim: tuple = None,
                    style: str = plot_style,
                    **kwargs):

            with plt.style.context(style):


                # Call the decorated function
                if args and hasattr(args[0], "__class__"):
                    figure = function(args[0], *args[1:], **kwargs)
                else:
                    figure = function(*args, **kwargs)

                # Apply axis customizations
                all_axes = axes if isinstance(axes, np.ndarray) else [axes]
                for ax in all_axes:
                    if xscale:
                        ax.set_xscale(xscale)
                    if yscale:
                        ax.set_yscale(yscale)
                    if xlim:
                        ax.set_xlim(*xlim)
                    if ylim:
                        ax.set_ylim(*ylim)

                if tight_layout:
                    figure.tight_layout()

                if save_as is not None:
                    figure.savefig(save_as, dpi=300)

                if show:
                    plt.show()

                return figure

        # merge docstrings (original + extra)
        if function.__doc__:
            wrapper.__doc__ = f"{function.__doc__}\n{extra_doc}"
        else:
            wrapper.__doc__ = extra_doc
        return wrapper


    return decorator


def add_inset(ax: plt.Axes, x_lim: tuple = None, y_lim: tuple = None, zoom: float = 2., loc: str = "upper center"):

    from mpl_toolkits.axes_grid1.inset_locator import zoomed_inset_axes, mark_inset

    axins = zoomed_inset_axes(ax, zoom=zoom, loc=loc)

    axins.set_xticks([])
    axins.set_yticks([])

    for line in ax.lines:
        axins.plot(
            *line.get_data(),
            color=line.get_color(),
            marker=line.get_marker(),
            label=line.get_label(),
            linestyle=line.get_linestyle(),
            linewidth=line.get_linewidth(),
        )

    for collection in ax.collections:
        verts = collection.get_paths()[0].vertices
        x_fill, y_fill = verts[:, 0], verts[:, 1]

        axins.fill_between(
            x_fill,
            0,
            y_fill,
            alpha=collection.get_alpha(),
            color=collection.get_facecolor(),
            label=collection.get_label()
        )

    mark_inset(ax, axins, loc1=2, loc2=4, fc="none", ec="0.5")

    if x_lim is not None:
        axins.set_xlim(x_lim)

    if y_lim is not None:
        axins.set_ylim(y_lim)

    return axins


import pytest
import numpy
import matplotlib.pyplot as plt

from MPSPlots import helper


def test_dummy():
    @helper.post_mpl_plot
    def _plot():
        figure, axes = plt.subplots(1, 1)
        x = numpy.arange(0, 100)
        y = 2 * x**2
        axes.plot(x, y)

        return figure

    _plot(show=True, yscale='log')


if __name__ == "__main__":
    pytest.main(["-W error", "-s", __file__])


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import MPSPlots
from sphinx_gallery.sorting import FileNameSortKey
from MPSPlots.styles import use_mpsplots_style
from MPSPlots.directories import project_path, doc_css_path

sys.path.insert(0, project_path)
sys.path.insert(0, os.path.join(project_path, "MPSPlots"))


def setup(app):
    app.add_css_file(str(doc_css_path))


autodoc_mock_imports = [
    'numpy',
    'matplotlib',
    'pyvista'
]

project = 'MPSPlots'
copyright = '2021, Martin Poinsinet de Sivry-Houle'
author = 'Martin Poinsinet de Sivry-Houle'
today_fmt = '%B %d, %Y'

version = MPSPlots.__version__


extensions = [
    'sphinx.ext.mathjax',
    'pyvista.ext.plot_directive',
    'sphinx_gallery.gen_gallery',
    'sphinx.ext.autodoc',
    'sphinx.ext.napoleon',
    'sphinx.ext.viewcode',
    'sphinx.ext.autosectionlabel',
    'sphinx.ext.intersphinx',
]

# Napoleon settings for docstrings
napoleon_google_docstring = False
napoleon_numpy_docstring = True


def reset_mpl(gallery_conf, fname):
    use_mpsplots_style()


try:
    import pyvista
    pyvista.start_xvfb()  # Works only on linux system!
except ImportError:
    print('Could not load pyvista library for 3D rendering')


sphinx_gallery_conf = {
    'examples_dirs': '../examples',
    'gallery_dirs': "gallery",
    'image_scrapers': ('matplotlib', 'pyvista'),
    'ignore_pattern': '/__',
    'plot_gallery': True,
    'thumbnail_size': [600, 600],
    'download_all_examples': False,
    'reset_modules': reset_mpl,
    'line_numbers': False,
    'remove_config_comments': True,
    'within_subsection_order': FileNameSortKey,
    'capture_repr': ('_repr_html_', '__repr__'),
    'nested_sections': True,
}

autodoc_default_options = {
    'members': False,
    'members-order': 'bysource',
    'undoc-members': False,
    'show-inheritance': True,
}


autosectionlabel_prefix_document = True
numpydoc_show_class_members = False
add_module_names = False


source_suffix = '.rst'
master_doc = 'index'
language = 'en'
highlight_language = 'python3'
html_theme = "pydata_sphinx_theme"

# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

exclude_trees = []
# default_role = "autolink"
pygments_style = "sphinx"

# -- Sphinx-gallery configuration --------------------------------------------
major, minor = version[:2]
binder_branch = f"v{major}.{minor}.x"

html_theme_options = {
    # Navigation bar
    "logo": {
        "alt_text": "MPSPlots's logo",
        "text": "MPSPlots",
        "link": "https://github.com/MartinPdeS/MPSPlots",
    },
    "icon_links": [
        {
            "name": "GitHub",
            "url": "https://github.com/MartinPdeS/MPSPlots",
            "icon": "fa-brands fa-github",
        },
        {
            "name": "PyPI",
            "url": "https://pypi.org/project/mpsplots/",
            "icon": "fa-solid fa-box",
        },
        {
            "name": "Anaconda",
            "url": "https://anaconda.org/MartinPdeS/mpsplots",
            "icon": "fa-brands fa-python",
        },
    ],
    "navbar_align": "left",
    "navbar_end": ["version-switcher", "navbar-icon-links"],
    "show_prev_next": False,
    "show_version_warning_banner": True,
    # Footer
    "footer_start": ["copyright"],
    "footer_end": ["sphinx-version", "theme-version"],
    # Other
    "pygment_light_style": "default",
    "pygment_dark_style": "github-dark",
}


htmlhelp_basename = 'MPSPlotsdoc'

latex_elements = {}

latex_documents = [
    (master_doc, 'MPSPlots.tex', 'MPSPlots Documentation',
     'Martin Poinsinet de Sivry-Houle', 'manual'),
]

man_pages = [
    (master_doc, 'pymiesim', 'MPSPlots Documentation',
     [author], 1)
]

texinfo_documents = [
    (master_doc, 'MPSPlots', 'MPSPlots Documentation',
     author, 'MPSPlots', 'One line description of project.',
     'Miscellaneous'),
]

epub_title = project

html_static_path = ['_static']
templates_path = ['_templates']
html_css_files = ['default.css']
epub_exclude_files = ['search.html']
